#include "slideparser.h"

#include "slideworld.h"
#include "sliderender.h"
#include "window.h"
#include "slideviewport.h"
#include "slidecamera.h"
#include "slidegroup.h"
#include "slideinstance.h"
#include "slidetransform.h"
#include "slidetranslate.h"
#include "slidescale.h"
#include "sliderotate.h"
#include "slideobject.h"
#include "slideface.h"
#include "slidepoint.h"
#include "slidesurface.h"

CSLIDEParser::CSLIDEParser()
{
  m_pCSLIDEWorld = NULL;
  m_pCSLIDERenderCurr = NULL;
  m_pCWindowCurr = NULL;
  m_pCSLIDEViewportCurr = NULL;
  m_pCSLIDECameraCurr = NULL;
  m_pCSLIDELightCurr = NULL;
  m_pCSLIDENodeCurr = NULL;
  m_pCSLIDEGroupCurr = NULL;
  m_pCSLIDEInstanceCurr = NULL;
  m_pCSLIDEObjectCurr = NULL;
  m_pCSLIDEFaceCurr = NULL;
  m_pCSLIDEPointCurr = NULL;
  m_pCSLIDESurfaceCurr = NULL;
  m_pCSLIDELookAtCurr = NULL;
}

CSLIDEParser::~CSLIDEParser()
{
}

VOID CSLIDEParser::Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
			CSLIDEParser **ppyy_CSLIDEParser, 
			YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
			CScanner **ppyy_CScanner,
			int *pyyleng,
			YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
			YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
			YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
			YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
			YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
			YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
			YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
			YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString)
{
  CParser::Init(pcFile, ppcDirectories, uiDirectories, 
		(CParser**)ppyy_CSLIDEParser, 
		pyylloc, pfyy_Parse,
		ppyy_CScanner,
		pyyleng,
		pfyy_BeginInitial,
		pfyy_GetCurrentBuffer,
		pfyy_CreateBuffer,
		pfyy_DeleteBuffer,
		pfyy_SwitchToBuffer,
		pfyy_SetValueInt,
		pfyy_SetValueFloat,
		pfyy_SetValueString);

  m_pCSLIDEWorld = CreateWorld();
  ASSERT( m_pCSLIDEWorld != NULL );
  m_pCSLIDERenderCurr = NULL;
  m_pCWindowCurr = NULL;
  m_pCSLIDEViewportCurr = NULL;
  m_pCSLIDECameraCurr = NULL;
  m_pCSLIDELightCurr = NULL;
  m_pCSLIDENodeCurr = NULL;
  m_pCSLIDEGroupCurr = NULL;
  m_pCSLIDEInstanceCurr = NULL;
  m_pCSLIDEObjectCurr = NULL;
  m_pCSLIDEFaceCurr = NULL;
  m_pCSLIDEPointCurr = NULL;
  m_pCSLIDESurfaceCurr = NULL;
  m_pCSLIDELookAtCurr = NULL;

  SetAST(m_pCSLIDEWorld);
}

VOID CSLIDEParser::Uninit()
{
  CParser::Uninit();
}

VOID CSLIDEParser::AddRender(CHAR *pcFile, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     CHAR *pcViewportID, CListInt *plistCameraPath, 
			     CHAR *pcRootID)
{
  CSLIDERender *pCSLIDERender;

  ASSERT( m_pCSLIDERenderCurr == NULL );

  pCSLIDERender = CreateRender(pcFile, uiLine, uiColumn, 
			       pcViewportID, plistCameraPath, 
			       pcRootID);
  ASSERT( pCSLIDERender != NULL );

  VERIFY(m_pCSLIDEWorld->InsertRender(pCSLIDERender));
  m_pCSLIDERenderCurr = pCSLIDERender;
}

VOID CSLIDEParser::EndRender()
{
  ASSERT( m_pCSLIDERenderCurr != NULL );
  m_pCSLIDERenderCurr = NULL;
}

VOID CSLIDEParser::SetRenderTclTag(CHAR *pcTclTag)
{
  ASSERT( m_pCSLIDERenderCurr != NULL );

  m_pCSLIDERenderCurr->SetTclTag(pcTclTag);
}

VOID CSLIDEParser::SetRenderLOD(DynInt *pdyniLOD)
{
  ASSERT( m_pCSLIDERenderCurr != NULL );

  m_pCSLIDERenderCurr->SetLOD(pdyniLOD);
  delete pdyniLOD;
}

VOID CSLIDEParser::AddRenderLightPath(CHAR *pcFile, UINT_32 uiLine, 
				      UINT_32 uiColumn, CListInt *plistLightPath,
				      DynInt *pdyniOnOff)
{
  CSLIDELightPath *pCSLIDELightPath;

  ASSERT( m_pCSLIDERenderCurr != NULL );

  pCSLIDELightPath = CreateLightPath(pcFile, uiLine, uiColumn, plistLightPath,
				     pdyniOnOff);
  ASSERT( pCSLIDELightPath == NULL );
}

VOID CSLIDEParser::AddWindow(CHAR *pcFile, UINT_32 uiLine, 
			      UINT_32 uiColumn,
			      CHAR *pcID)
{
  CWindow *pCWindow;

  ASSERT( m_pCWindowCurr == NULL );

  pCWindow = m_pCSLIDEWorld->PeekWindow(pcID);
  if ( pCWindow != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtWindow],
		  pcID, pCWindow->GetFileName(), 
		  pCWindow->GetFileLine(), 
		  pCWindow->GetFileColumn());
    }

  pCWindow = CreateWindow(pcFile, uiLine, uiColumn, pcID);
  ASSERT( pCWindow != NULL );

  VERIFY(m_pCSLIDEWorld->InsertWindow(pCWindow));
  m_pCWindowCurr = pCWindow;
}

VOID CSLIDEParser::EndWindow()
{
  ASSERT( m_pCWindowCurr != NULL );
  m_pCWindowCurr = NULL;
}

VOID CSLIDEParser::SetWindowOrigin(FLOAT fX, FLOAT fY)
{
  ASSERT( m_pCWindowCurr != NULL );

  m_pCWindowCurr->SetOrigin(fX, fY);
}

VOID CSLIDEParser::SetWindowSize(FLOAT fX, FLOAT fY)
{
  ASSERT( m_pCWindowCurr != NULL );

  m_pCWindowCurr->SetSize(fX, fY);
}

VOID CSLIDEParser::SetWindowBackground(DynVector *pdynv)
{
  ASSERT( m_pCWindowCurr != NULL );

   m_pCWindowCurr->SetBackground(pdynv);
}

VOID CSLIDEParser::AddViewport(CHAR *pcFile, UINT_32 uiLine, 
			      UINT_32 uiColumn,
			      CHAR *pcID, CHAR *pcWindowID)
{
  CSLIDEViewport *pCSLIDEViewport;

  ASSERT( m_pCSLIDEViewportCurr == NULL );

  pCSLIDEViewport = m_pCSLIDEWorld->PeekViewport(pcID);
  if ( pCSLIDEViewport != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtViewport],
		  pcID, pCSLIDEViewport->GetFileName(), 
		  pCSLIDEViewport->GetFileLine(), 
		  pCSLIDEViewport->GetFileColumn());
    }

  pCSLIDEViewport = CreateViewport(pcFile, uiLine, uiColumn, pcID,
				   pcWindowID);
  ASSERT( pCSLIDEViewport != NULL );

  VERIFY(m_pCSLIDEWorld->InsertViewport(pCSLIDEViewport));
  m_pCSLIDEViewportCurr = pCSLIDEViewport;
}

VOID CSLIDEParser::EndViewport()
{
  ASSERT( m_pCSLIDEViewportCurr != NULL );
  m_pCSLIDEViewportCurr = NULL;
}

VOID CSLIDEParser::SetViewportOrigin(DynVector *pdynv)
{
  ASSERT( m_pCSLIDEViewportCurr != NULL );

  m_pCSLIDEViewportCurr->SetOrigin(pdynv);
  delete pdynv;
}

VOID CSLIDEParser::SetViewportSize(DynVector *pdynv)
{
  ASSERT( m_pCSLIDEViewportCurr != NULL );

  m_pCSLIDEViewportCurr->SetSize(pdynv);
  delete pdynv;
}

VOID CSLIDEParser::AddCamera(CHAR *pcFile, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     CHAR *pcID)
{
  CSLIDENode *pCSLIDENode;
  CSLIDECamera *pCSLIDECamera;

  ASSERT( m_pCSLIDECameraCurr == NULL );

  pCSLIDENode = m_pCSLIDEWorld->PeekNode(pcID);
  if ( pCSLIDENode != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtCamera],
		  pcID, pCSLIDENode->GetFileName(), 
		  pCSLIDENode->GetFileLine(), 
		  pCSLIDENode->GetFileColumn());
    }

  pCSLIDECamera = CreateCamera(pcFile, uiLine, uiColumn, pcID);
  ASSERT( pCSLIDECamera != NULL );

  VERIFY(m_pCSLIDEWorld->InsertNode(pCSLIDECamera));
  m_pCSLIDECameraCurr = pCSLIDECamera;
}

VOID CSLIDEParser::EndCamera()
{
  ASSERT( m_pCSLIDECameraCurr != NULL );
  m_pCSLIDECameraCurr = NULL;
}

VOID CSLIDEParser::SetCameraProjection(DynInt *pdyniProjection)
{
  ASSERT( m_pCSLIDECameraCurr != NULL );

  m_pCSLIDECameraCurr->SetProjection(pdyniProjection);
  delete pdyniProjection;
}

VOID CSLIDEParser::SetCameraFrustum(DynVector *pdynvMin, 
				    DynVector *pdynvMax)
{
  ASSERT( m_pCSLIDECameraCurr != NULL );

  m_pCSLIDECameraCurr->SetFrustumMin(pdynvMin);
  m_pCSLIDECameraCurr->SetFrustumMax(pdynvMax);
  delete pdynvMin;
  delete pdynvMax;
}

VOID CSLIDEParser::AddLight(CHAR * /*pcFile*/, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     CHAR *pcID)
{
  CSLIDENode *pCSLIDENode;

  ASSERT( m_pCSLIDELightCurr == NULL );

  pCSLIDENode = m_pCSLIDEWorld->PeekNode(pcID);
  if ( pCSLIDENode != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtLight],
		  pcID, pCSLIDENode->GetFileName(), 
		  pCSLIDENode->GetFileLine(), 
		  pCSLIDENode->GetFileColumn());
    }
}

VOID CSLIDEParser::EndLight()
{
}

VOID CSLIDEParser::SetLightType(DynInt *pdyniType)
{
  delete pdyniType;
}

VOID CSLIDEParser::SetLightColor(DynVector *pdynvColor)
{
  delete pdynvColor;
}

VOID CSLIDEParser::SetLightDeadDistance(DynFloat *pdynfDeadDistance)
{
  delete pdynfDeadDistance;
}

VOID CSLIDEParser::SetLightFallOff(DynFloat *pdynfFallOff)
{
  delete pdynfFallOff;
}

VOID CSLIDEParser::SetLightAngularFallOff(DynFloat *pdynfAngularFallOff)
{
  delete pdynfAngularFallOff;
}

VOID CSLIDEParser::SetNodeSurfaceID(CHAR *pcSurfaceID)
{
  ASSERT( m_pCSLIDENodeCurr != NULL );

  m_pCSLIDENodeCurr->SetSurfaceID(pcSurfaceID);
}

VOID CSLIDEParser::SetNodeShading(DynInt *pdyniShading)
{
  ASSERT( m_pCSLIDENodeCurr != NULL );

  m_pCSLIDENodeCurr->SetShading(pdyniShading);
  delete pdyniShading;
}

VOID CSLIDEParser::SetNodeLOD(DynInt *pdyniLOD)
{
  ASSERT( m_pCSLIDENodeCurr != NULL );

  m_pCSLIDENodeCurr->SetLOD(pdyniLOD);
  delete pdyniLOD;
}

VOID CSLIDEParser::AddGroup(CHAR *pcFile, UINT_32 uiLine, 
			    UINT_32 uiColumn,
			    CHAR *pcID)
{
  CSLIDENode *pCSLIDENode;

  ASSERT( m_pCSLIDENodeCurr == NULL );
  ASSERT( m_pCSLIDEGroupCurr == NULL );

  pCSLIDENode = m_pCSLIDEWorld->PeekNode(pcID);
  if ( pCSLIDENode != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtObject],
		  pcID, pCSLIDENode->GetFileName(), 
		  pCSLIDENode->GetFileLine(), 
		  pCSLIDENode->GetFileColumn());
    }

  pCSLIDENode = CreateGroup(pcFile, uiLine, uiColumn, pcID);
  ASSERT( pCSLIDENode != NULL );

  VERIFY(m_pCSLIDEWorld->InsertNode(pCSLIDENode));
  m_pCSLIDENodeCurr = pCSLIDENode;
  m_pCSLIDEGroupCurr = (CSLIDEGroup*)m_pCSLIDENodeCurr;
}

VOID CSLIDEParser::EndGroup()
{
  ASSERT( m_pCSLIDEGroupCurr != NULL );
  ASSERT( m_pCSLIDENodeCurr == m_pCSLIDEGroupCurr );
  m_pCSLIDEGroupCurr = NULL;
  m_pCSLIDENodeCurr = NULL;
}

VOID CSLIDEParser::AddInstance(CHAR *pcFile, UINT_32 uiLine, 
			       UINT_32 uiColumn, CHAR *pcChildID)
{
  CSLIDEInstance *pCSLIDEInstance;

  ASSERT( m_pCSLIDEGroupCurr != NULL );
  ASSERT( m_pCSLIDEGroupCurr == m_pCSLIDENodeCurr );
  ASSERT( m_pCSLIDEInstanceCurr == NULL );

  pCSLIDEInstance = CreateInstance(pcFile, uiLine, uiColumn, pcChildID);
  ASSERT( pCSLIDEInstance != NULL );

  VERIFY(m_pCSLIDEWorld->InsertInstance(pCSLIDEInstance));
  m_pCSLIDEGroupCurr->InsertInstance(pCSLIDEInstance);
  m_pCSLIDEInstanceCurr = pCSLIDEInstance;
  m_pCSLIDENodeCurr = m_pCSLIDEInstanceCurr;
}

VOID CSLIDEParser::EndInstance()
{
  ASSERT( m_pCSLIDEInstanceCurr != NULL );
  ASSERT( m_pCSLIDEInstanceCurr == m_pCSLIDENodeCurr );
  ASSERT( m_pCSLIDEGroupCurr != NULL );
  m_pCSLIDEInstanceCurr = NULL;
  m_pCSLIDENodeCurr = m_pCSLIDEGroupCurr;
}

VOID CSLIDEParser::SetInstanceID(CHAR *pcID)
{
  ASSERT( m_pCSLIDEInstanceCurr != NULL );

  m_pCSLIDEInstanceCurr->SetID(pcID);
}

VOID CSLIDEParser::AddTranslate(CHAR *pcFile, UINT_32 uiLine, 
				UINT_32 uiColumn,
				DynVector *pdynv)
{
  CSLIDETransform *pCSLIDETransform;

  ASSERT( m_pCSLIDEInstanceCurr != NULL );

  pCSLIDETransform = CreateTranslate(pcFile, uiLine, uiColumn, 
				     pdynv);
  ASSERT( pCSLIDETransform != NULL );

  VERIFY(m_pCSLIDEWorld->InsertTransform(pCSLIDETransform));
  m_pCSLIDEInstanceCurr->InsertTransform(pCSLIDETransform);
}

VOID CSLIDEParser::AddScale(CHAR *pcFile, UINT_32 uiLine, 
			    UINT_32 uiColumn,
			    DynVector *pdynv)
{
  CSLIDETransform *pCSLIDETransform;

  ASSERT( m_pCSLIDEInstanceCurr != NULL );

  pCSLIDETransform = CreateScale(pcFile, uiLine, uiColumn, 
				 pdynv);
  ASSERT( pCSLIDETransform != NULL );

  VERIFY(m_pCSLIDEWorld->InsertTransform(pCSLIDETransform));
  m_pCSLIDEInstanceCurr->InsertTransform(pCSLIDETransform);
}

VOID CSLIDEParser::AddRotate(CHAR *pcFile, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     DynVector *pdynv, DynFloat *pdynfAngle)
{
  CSLIDETransform *pCSLIDETransform;

  ASSERT( m_pCSLIDEInstanceCurr != NULL );

  pCSLIDETransform = CreateRotate(pcFile, uiLine, uiColumn, 
				  pdynv, pdynfAngle);
  ASSERT( pCSLIDETransform != NULL );

  VERIFY(m_pCSLIDEWorld->InsertTransform(pCSLIDETransform));
  m_pCSLIDEInstanceCurr->InsertTransform(pCSLIDETransform);
}

VOID CSLIDEParser::AddLookAt(CHAR * /*pcFile*/, UINT_32 uiLine, 
			     UINT_32 uiColumn)
{

      HandleError(uiLine, uiColumn, "\"lookat\" transforms are not supported in this assignment's parser.");
}

VOID CSLIDEParser::EndLookAt()
{
}

VOID CSLIDEParser::SetLookAtEye(DynVector *pdynv)
{
  delete pdynv;
}

VOID CSLIDEParser::SetLookAtTarget(DynVector *pdynv)
{
  delete pdynv;
}

VOID CSLIDEParser::SetLookAtUp(DynVector *pdynv)
{
  delete pdynv;
}

#ifdef SLIDE_HAS_GENERAL_MATRICES
VOID CSLIDEParser::AddGeneral(CHAR *pcFile, UINT_32 uiLine, 
			      UINT_32 uiColumn,
			      FLOAT f0, FLOAT f1, FLOAT f2, FLOAT f3, 
			      FLOAT f4, FLOAT f5, FLOAT f6, FLOAT f7, 
			      FLOAT f8, FLOAT f9, FLOAT f10, FLOAT f11, 
			      FLOAT f12, FLOAT f13, FLOAT f14, FLOAT f15) 
{
  CSLIDETransform *pCSLIDETransform;
  Matrix m;

  ASSERT( m_pCSLIDEInstanceCurr != NULL );

  // Row Major Order -- Column Matrices
  /*
  m.Set(0, 0, f0);  m.Set(0, 1, f1);  m.Set(0, 2, f2);  m.Set(0, 3, f3); 
  m.Set(1, 0, f4);  m.Set(1, 1, f5);  m.Set(1, 2, f6);  m.Set(1, 3, f7); 
  m.Set(2, 0, f8);  m.Set(2, 1, f9);  m.Set(2, 2, f10); m.Set(2, 3, f11); 
  m.Set(3, 0, f12); m.Set(3, 1, f13); m.Set(3, 2, f14); m.Set(3, 3, f15); 
  */

  // Column Major Order -- Column Matrices
  m.Set(0, 0, f0);  m.Set(0, 1, f4);  m.Set(0, 2, f8);  m.Set(0, 3, f12); 
  m.Set(1, 0, f1);  m.Set(1, 1, f5);  m.Set(1, 2, f9);  m.Set(1, 3, f13); 
  m.Set(2, 0, f2);  m.Set(2, 1, f6);  m.Set(2, 2, f10); m.Set(2, 3, f14); 
  m.Set(3, 0, f3);  m.Set(3, 1, f7);  m.Set(3, 2, f11); m.Set(3, 3, f15); 

  pCSLIDETransform = CreateGeneral(pcFile, uiLine, uiColumn, m);
  ASSERT( pCSLIDETransform != NULL );

  VERIFY(m_pCSLIDEWorld->InsertTransform(pCSLIDETransform));
  m_pCSLIDEInstanceCurr->InsertTransform(pCSLIDETransform);
}
#endif // SLIDE_HAS_GENERAL_MATRICES

VOID CSLIDEParser::AddObject(CHAR *pcFile, UINT_32 uiLine, 
			     UINT_32 uiColumn,
			     CHAR *pcID, 
			     CListInt *plistFaceIDs)
{
  CSLIDENode *pCSLIDENode;

  ASSERT( m_pCSLIDENodeCurr == NULL );
  ASSERT( m_pCSLIDEObjectCurr == NULL );

  pCSLIDENode = m_pCSLIDEWorld->PeekNode(pcID);
  if ( pCSLIDENode != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtObject],
		  pcID, pCSLIDENode->GetFileName(), 
		  pCSLIDENode->GetFileLine(), 
		  pCSLIDENode->GetFileColumn());
    }

  pCSLIDENode = CreateObject(pcFile, uiLine, uiColumn,
			     pcID, plistFaceIDs);
  ASSERT( pCSLIDENode != NULL );

  VERIFY(m_pCSLIDEWorld->InsertNode(pCSLIDENode));
  m_pCSLIDENodeCurr = pCSLIDENode;
  m_pCSLIDEObjectCurr = (CSLIDEObject*)m_pCSLIDENodeCurr;
}

VOID CSLIDEParser::EndObject()
{
  ASSERT( m_pCSLIDEObjectCurr != NULL );
  ASSERT( m_pCSLIDENodeCurr == m_pCSLIDEObjectCurr );
  m_pCSLIDEObjectCurr = NULL;
  m_pCSLIDENodeCurr = NULL;  
}

VOID CSLIDEParser::SetObjectSolid(DynInt *pdyniSolid)
{
  ASSERT( m_pCSLIDEObjectCurr != NULL );

  m_pCSLIDEObjectCurr->SetSolid(pdyniSolid);
  delete pdyniSolid;
}

VOID CSLIDEParser::AddFace(CHAR *pcFile, UINT_32 uiLine, 
			   UINT_32 uiColumn,
			   CHAR *pcID, 
			   CListInt *plistPointIDs)
{
  CSLIDEFace *pCSLIDEFace;

  ASSERT( m_pCSLIDEFaceCurr == NULL );

  pCSLIDEFace = m_pCSLIDEWorld->PeekFace(pcID);
  if ( pCSLIDEFace != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld",  
		  gcppcSLIDEReservedWords[sliderwtFace],
		  pcID, pCSLIDEFace->GetFileName(), 
		  pCSLIDEFace->GetFileLine(), 
		  pCSLIDEFace->GetFileColumn());
    }

  pCSLIDEFace = CreateFace(pcFile, uiLine, uiColumn, 
			   pcID, plistPointIDs);
  ASSERT( pCSLIDEFace != NULL );

  VERIFY(m_pCSLIDEWorld->InsertFace(pCSLIDEFace));
  m_pCSLIDEFaceCurr = pCSLIDEFace;
}

VOID CSLIDEParser::EndFace()
{
  ASSERT( m_pCSLIDEFaceCurr != NULL );
  m_pCSLIDEFaceCurr = NULL;
}

VOID CSLIDEParser::SetFaceSurfaceID(CHAR *pcSurfaceID)
{
  ASSERT( m_pCSLIDEFaceCurr != NULL );

  m_pCSLIDEFaceCurr->SetSurfaceID(pcSurfaceID);
}

VOID CSLIDEParser::AddPoint(CHAR *pcFile, UINT_32 uiLine, 
			    UINT_32 uiColumn, CHAR *pcID,
			    DynPoint *pdynpt)
{
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( m_pCSLIDEPointCurr == NULL );

  pCSLIDEPoint = m_pCSLIDEWorld->PeekPoint(pcID);
  if ( pCSLIDEPoint != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtPoint],
		  pcID, pCSLIDEPoint->GetFileName(), 
		  pCSLIDEPoint->GetFileLine(), 
		  pCSLIDEPoint->GetFileColumn());
    }

  pCSLIDEPoint = CreatePoint(pcFile, uiLine, uiColumn,
			     pcID, pdynpt);
  ASSERT( pCSLIDEPoint != NULL );

  VERIFY(m_pCSLIDEWorld->InsertPoint(pCSLIDEPoint));
  m_pCSLIDEPointCurr = pCSLIDEPoint;
}

VOID CSLIDEParser::EndPoint()
{
  ASSERT( m_pCSLIDEPointCurr != NULL );
  m_pCSLIDEPointCurr = NULL;
}

VOID CSLIDEParser::SetPointSurfaceID(CHAR *pcSurfaceID)
{
  ASSERT( m_pCSLIDEPointCurr != NULL );

  m_pCSLIDEPointCurr->SetSurfaceID(pcSurfaceID);
}

VOID CSLIDEParser::SetPointNormal(DynVector *pdynv)
{
  ASSERT( m_pCSLIDEPointCurr != NULL );

  m_pCSLIDEPointCurr->SetNormal(pdynv);
  delete pdynv;
}

VOID CSLIDEParser::SetPointTexture(DynPoint *pdynpt)
{
  ASSERT( m_pCSLIDEPointCurr != NULL );

  m_pCSLIDEPointCurr->SetTexture(pdynpt);
  delete pdynpt;
}

VOID CSLIDEParser::AddSurface(CHAR *pcFile, UINT_32 uiLine, 
			      UINT_32 uiColumn,
			      CHAR *pcID)
{
  CSLIDESurface *pCSLIDESurface;

  ASSERT( m_pCSLIDESurfaceCurr == NULL );

  pCSLIDESurface = m_pCSLIDEWorld->PeekSurface(pcID);
  if ( pCSLIDESurface != NULL )
    {
      HandleError(uiLine, uiColumn, "%s \"%s\" previously defined at %s: %ld: %ld", 
		  gcppcSLIDEReservedWords[sliderwtSurface],
		  pcID, pCSLIDESurface->GetFileName(), 
		  pCSLIDESurface->GetFileLine(), 
		  pCSLIDESurface->GetFileColumn());
    }

  pCSLIDESurface = CreateSurface(pcFile, uiLine, uiColumn, pcID);
  ASSERT( pCSLIDESurface != NULL );

  VERIFY(m_pCSLIDEWorld->InsertSurface(pCSLIDESurface));
  m_pCSLIDESurfaceCurr = pCSLIDESurface;
}

VOID CSLIDEParser::EndSurface()
{
  ASSERT( m_pCSLIDESurfaceCurr != NULL );
  m_pCSLIDESurfaceCurr = NULL;
}

VOID CSLIDEParser::SetSurfaceColor(DynVector *pdynv)
{
  ASSERT( m_pCSLIDESurfaceCurr != NULL );

  m_pCSLIDESurfaceCurr->SetColor(pdynv);
  delete pdynv;
}

VOID CSLIDEParser::SetSurfaceReflectivity(DynVector *pdynv)
{
  ASSERT( m_pCSLIDESurfaceCurr != NULL );

  m_pCSLIDESurfaceCurr->SetReflectivity(pdynv);
  delete pdynv;
}

VOID CSLIDEParser::SetSurfaceExponent(DynFloat *pdynfExponent)
{
  ASSERT( m_pCSLIDESurfaceCurr != NULL );

  m_pCSLIDESurfaceCurr->SetExponent(pdynfExponent);
  delete pdynfExponent;
}

VOID CSLIDEParser::SetSurfaceMetallic(DynFloat *pdynfMetallic)
{
  ASSERT( m_pCSLIDESurfaceCurr != NULL );

  m_pCSLIDESurfaceCurr->SetMetallic(pdynfMetallic);
  delete pdynfMetallic;
}

VOID CSLIDEParser::AddGlobalInitTclBlock(CHAR *pcTclBlock)
{
  VERIFY(m_pCSLIDEWorld->InsertGlobalInitTclBlock(pcTclBlock));
}

VOID CSLIDEParser::AddGlobalUpdateTclBlock(CHAR *pcTclBlock)
{
  VERIFY(m_pCSLIDEWorld->InsertGlobalUpdateTclBlock(pcTclBlock));
}

CListInt *CSLIDEParser::CreateIDList()
{
  return CreateList();
}

CListInt *CSLIDEParser::InsertIDList(CListInt *pCListInt, CHAR *pcID)
{
  return InsertList(pCListInt, pcID);
}

CSLIDEWorld *CSLIDEParser::CreateWorld()
{
  CSLIDEWorld *pCSLIDEWorld;

  pCSLIDEWorld = new CSLIDEWorld();
  ASSERT( pCSLIDEWorld != NULL );
  pCSLIDEWorld->Init();

  return pCSLIDEWorld;
}

CSLIDERender *CSLIDEParser::CreateRender(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 CHAR *pcViewportID, 
					 CListInt *plistCameraPath, 
					 CHAR *pcRootID)
{
  CSLIDECameraPath *pCSLIDECameraPath;
  CSLIDERender *pCSLIDERender;

  pCSLIDECameraPath = new CSLIDECameraPath();
  ASSERT( pCSLIDECameraPath != NULL );
  pCSLIDECameraPath->Init(pcFile, uiLine, uiColumn, plistCameraPath);

  pCSLIDERender = new CSLIDERender();
  ASSERT( pCSLIDERender != NULL );
  pCSLIDERender->Init(pcFile, uiLine, uiColumn, pcViewportID, 
		      pCSLIDECameraPath, pcRootID);

  return pCSLIDERender;
}

CSLIDELightPath *CSLIDEParser::CreateLightPath(CHAR * /*pcFile*/, UINT_32 /*uiLine*/, 
					       UINT_32 /*uiColumn*/,
					       CListInt *plistLightPath,
					       DynInt *pdyniOnOff)
{
  if ( plistLightPath != NULL )
    {
      delete plistLightPath;
    }

  if ( pdyniOnOff != NULL )
    {
      delete pdyniOnOff;
    }

  return NULL;
}

CWindow *CSLIDEParser::CreateWindow(CHAR *pcFile, UINT_32 uiLine, 
				    UINT_32 uiColumn,
				    CHAR *pcID)
{
  CWindow *pCWindow;

  pCWindow = new CWindow();
  ASSERT( pCWindow != NULL );
  pCWindow->Init(pcFile, uiLine, uiColumn, pcID);

  return pCWindow;
}

CSLIDEViewport *CSLIDEParser::CreateViewport(CHAR *pcFile, 
					     UINT_32 uiLine, 
					     UINT_32 uiColumn,
					     CHAR *pcID, 
					     CHAR *pcWindowID)
{
  CSLIDEViewport *pCSLIDEViewport;

  pCSLIDEViewport = new CSLIDEViewport();
  ASSERT( pCSLIDEViewport != NULL );
  pCSLIDEViewport->Init(pcFile, uiLine, uiColumn, pcID, pcWindowID);

  return pCSLIDEViewport;
}

CSLIDECamera *CSLIDEParser::CreateCamera(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 CHAR *pcID)
{
  CSLIDECamera *pCSLIDECamera;

  pCSLIDECamera = new CSLIDECamera();
  ASSERT( pCSLIDECamera != NULL );
  pCSLIDECamera->Init(pcFile, uiLine, uiColumn, pcID);

  return pCSLIDECamera;
}

CSLIDELight *CSLIDEParser::CreateLight(CHAR * /*pcFile*/, UINT_32 /*uiLine*/, 
					 UINT_32 /*uiColumn*/,
					 CHAR */*pcID*/)
{
  return NULL;
}

CSLIDEGroup *CSLIDEParser::CreateGroup(CHAR *pcFile, UINT_32 uiLine, 
				       UINT_32 uiColumn,
				       CHAR *pcID)
{
  CSLIDEGroup *pCSLIDEGroup;

  pCSLIDEGroup = new CSLIDEGroup();
  ASSERT( pCSLIDEGroup != NULL );
  pCSLIDEGroup->Init(pcFile, uiLine, uiColumn, pcID);

  return pCSLIDEGroup;
}

CSLIDEInstance *CSLIDEParser::CreateInstance(CHAR *pcFile, 
					     UINT_32 uiLine, 
					     UINT_32 uiColumn,
					     CHAR *pcChildID)
{
  CSLIDEInstance *pCSLIDEInstance;

  pCSLIDEInstance = new CSLIDEInstance();
  ASSERT( pCSLIDEInstance != NULL );
  pCSLIDEInstance->Init(pcFile, uiLine, uiColumn, pcChildID);

  return pCSLIDEInstance;
}

CSLIDETranslate *CSLIDEParser::CreateTranslate(CHAR *pcFile, 
					       UINT_32 uiLine,
					       UINT_32 uiColumn,
					       DynVector *pdynv)
{
  CSLIDETranslate *pCSLIDETranslate;

  pCSLIDETranslate = new CSLIDETranslate();
  ASSERT( pCSLIDETranslate != NULL );
  pCSLIDETranslate->Init(pcFile, uiLine, uiColumn, pdynv);

  delete pdynv;

  return pCSLIDETranslate;
}

CSLIDEScale *CSLIDEParser::CreateScale(CHAR *pcFile, UINT_32 uiLine, 
				       UINT_32 uiColumn,
				       DynVector *pdynv)
{
  CSLIDEScale *pCSLIDEScale;

  pCSLIDEScale = new CSLIDEScale();
  ASSERT( pCSLIDEScale != NULL );
  pCSLIDEScale->Init(pcFile, uiLine, uiColumn, pdynv);

  delete pdynv;

  return pCSLIDEScale;
}

CSLIDERotate *CSLIDEParser::CreateRotate(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 DynVector *pdynvAxis,
					 DynFloat *pdynfAngle)
{
  CSLIDERotate *pCSLIDERotate;

  pCSLIDERotate = new CSLIDERotate();
  ASSERT( pCSLIDERotate != NULL );

  pdynfAngle->SetValue((FLOAT)DEGTORAD(pdynfAngle->GetValue()));
  pCSLIDERotate->Init(pcFile, uiLine, uiColumn, pdynvAxis,
		      pdynfAngle);

  delete pdynvAxis;
  delete pdynfAngle;

  return pCSLIDERotate;
}

CSLIDELookAt *CSLIDEParser::CreateLookAt(CHAR * /*pcFile*/, UINT_32 /*uiLine*/, 
					 UINT_32 /*uiColumn*/)
{
  return NULL;
}

/*
CSLIDETransform *CSLIDEParser::CreateGeneral(CHAR *pcFile, 
					     UINT_32 uiLine, 
					     UINT_32 uiColumn,
					     const Matrix &m)
{
  CSLIDETransform *pCSLIDETransform;

  pCSLIDETransform = new CSLIDETransform();
  ASSERT( pCSLIDETransform != NULL );
  pCSLIDETransform->Init(pcFile, uiLine, uiColumn);
  pCSLIDETransform->SetMatrix(m);

  return pCSLIDETransform;
}
*/
CSLIDEObject *CSLIDEParser::CreateObject(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 CHAR *pcID, 
					 CListInt *plistFaceIDs)
{
  CSLIDEObject *pCSLIDEObject;

  pCSLIDEObject = new CSLIDEObject();
  ASSERT( pCSLIDEObject != NULL );
  pCSLIDEObject->Init(pcFile, uiLine, uiColumn, pcID,
		      plistFaceIDs);

  return pCSLIDEObject;
}

CSLIDEFace *CSLIDEParser::CreateFace(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcID, 
				     CListInt *plistPointIDs)
{
  CSLIDEFace *pCSLIDEFace;

  pCSLIDEFace = new CSLIDEFace();
  ASSERT( pCSLIDEFace != NULL );
  pCSLIDEFace->Init(pcFile, uiLine, uiColumn, pcID, plistPointIDs);

  return pCSLIDEFace;
}

CSLIDEPoint *CSLIDEParser::CreatePoint(CHAR *pcFile, UINT_32 uiLine, 
				       UINT_32 uiColumn,
				       CHAR *pcID, 
				       DynPoint *pdynpt)
{
  CSLIDEPoint *pCSLIDEPoint;

  pCSLIDEPoint = new CSLIDEPoint();
  ASSERT( pCSLIDEPoint != NULL );
  pCSLIDEPoint->Init(pcFile, uiLine, uiColumn, pcID, pdynpt);

  delete pdynpt;

  return pCSLIDEPoint;
}

CSLIDESurface *CSLIDEParser::CreateSurface(CHAR *pcFile, UINT_32 uiLine, 
					   UINT_32 uiColumn,
					   CHAR *pcID)
{
  CSLIDESurface *pCSLIDESurface;

  pCSLIDESurface = new CSLIDESurface();
  ASSERT( pCSLIDESurface != NULL );
  pCSLIDESurface->Init(pcFile, uiLine, uiColumn, pcID);

  return pCSLIDESurface;
}

CListInt *CSLIDEParser::CreateList()
{
  CListInt *pCListInt;

  pCListInt = new CListInt();
  ASSERT( pCListInt != NULL );

  pCListInt->Init();

  return pCListInt;
}

CListInt *CSLIDEParser::InsertList(CListInt *pCListInt, VOID *pvData)
{
  ASSERT( pCListInt != NULL );

  pCListInt->InsertLast(pvData, 0);

  return pCListInt;
}


